@@ -3,6 +3,8 @@ from urlparse import urlparse, urlunparse, urlsplit
 import sys
 import os
 import re
+import mimetypes
+from copy import copy
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -54,6 +56,10 @@ class ClientHandler(BaseHandler):
     Uses the WSGI interface to compose requests, but returns
     the raw HttpResponse object
     """
+    def __init__(self, enforce_csrf_checks=True, *args, **kwargs):
+        self.enforce_csrf_checks = enforce_csrf_checks
+        super(ClientHandler, self).__init__(*args, **kwargs)
+
     def __call__(self, environ):
         from django.conf import settings
         from django.core import signals
@@ -66,6 +72,11 @@ class ClientHandler(BaseHandler):
         signals.request_started.send(sender=self.__class__)
         try:
             request = WSGIRequest(environ)
+            # sneaky little hack so that we can easily get round
+            # CsrfViewMiddleware.  This makes life easier, and is probably
+            # required for backwards compatibility with external tests against
+            # admin views.
+            request._dont_enforce_csrf_checks = not self.enforce_csrf_checks
             response = self.get_response(request)
 
             # Apply response middleware.
@@ -82,9 +93,12 @@ class ClientHandler(BaseHandler):
 def store_rendered_templates(store, signal, sender, template, context, **kwargs):
     """
     Stores templates and contexts that are rendered.
+
+    The context is copied so that it is an accurate representation at the time
+    of rendering.
     """
     store.setdefault('template', []).append(template)
-    store.setdefault('context', ContextList()).append(context)
+    store.setdefault('context', ContextList()).append(copy(context))
 
 def encode_multipart(boundary, data):
     """
@@ -133,11 +147,14 @@ def encode_multipart(boundary, data):
 
 def encode_file(boundary, key, file):
     to_str = lambda s: smart_str(s, settings.DEFAULT_CHARSET)
+    content_type = mimetypes.guess_type(file.name)[0]
+    if content_type is None:
+        content_type = 'application/octet-stream'
     return [
         '--' + boundary,
         'Content-Disposition: form-data; name="%s"; filename="%s"' \
             % (to_str(key), to_str(os.path.basename(file.name))),
-        'Content-Type: application/octet-stream',
+        'Content-Type: %s' % content_type,
         '',
         file.read()
     ]
@@ -160,8 +177,8 @@ class Client(object):
     contexts and templates produced by a view, rather than the
     HTML rendered to the end-user.
     """
-    def __init__(self, **defaults):
-        self.handler = ClientHandler()
+    def __init__(self, enforce_csrf_checks=False, **defaults):
+        self.handler = ClientHandler(enforce_csrf_checks)
         self.defaults = defaults
         self.cookies = SimpleCookie()
         self.exc_info = None
@@ -185,6 +202,13 @@ class Client(object):
         return {}
     session = property(_session)
 
+    def _get_path(self, parsed):
+        # If there are parameters, add them
+        if parsed[3]:
+             return urllib.unquote(parsed[2] + ";" + parsed[3])
+        else:
+             return urllib.unquote(parsed[2])
+
     def request(self, **request):
         """
         The master request method. Composes the environment dictionary
@@ -275,7 +299,7 @@ class Client(object):
         parsed = urlparse(path)
         r = {
             'CONTENT_TYPE':    'text/html; charset=utf-8',
-            'PATH_INFO':       urllib.unquote(parsed[2]),
+            'PATH_INFO':       self._get_path(parsed),
             'QUERY_STRING':    urlencode(data, doseq=True) or parsed[4],
             'REQUEST_METHOD': 'GET',
             'wsgi.input':      FakePayload('')
@@ -284,7 +308,7 @@ class Client(object):
 
         response = self.request(**r)
         if follow:
-            response = self._handle_redirects(response)
+            response = self._handle_redirects(response, **extra)
         return response
 
     def post(self, path, data={}, content_type=MULTIPART_CONTENT,
@@ -307,7 +331,7 @@ class Client(object):
         r = {
             'CONTENT_LENGTH': len(post_data),
             'CONTENT_TYPE':   content_type,
-            'PATH_INFO':      urllib.unquote(parsed[2]),
+            'PATH_INFO':      self._get_path(parsed),
             'QUERY_STRING':   parsed[4],
             'REQUEST_METHOD': 'POST',
             'wsgi.input':     FakePayload(post_data),
@@ -316,7 +340,7 @@ class Client(object):
 
         response = self.request(**r)
         if follow:
-            response = self._handle_redirects(response)
+            response = self._handle_redirects(response, **extra)
         return response
 
     def head(self, path, data={}, follow=False, **extra):
@@ -326,7 +350,7 @@ class Client(object):
         parsed = urlparse(path)
         r = {
             'CONTENT_TYPE':    'text/html; charset=utf-8',
-            'PATH_INFO':       urllib.unquote(parsed[2]),
+            'PATH_INFO':       self._get_path(parsed),
             'QUERY_STRING':    urlencode(data, doseq=True) or parsed[4],
             'REQUEST_METHOD': 'HEAD',
             'wsgi.input':      FakePayload('')
@@ -335,7 +359,7 @@ class Client(object):
 
         response = self.request(**r)
         if follow:
-            response = self._handle_redirects(response)
+            response = self._handle_redirects(response, **extra)
         return response
 
     def options(self, path, data={}, follow=False, **extra):
@@ -344,7 +368,7 @@ class Client(object):
         """
         parsed = urlparse(path)
         r = {
-            'PATH_INFO':       urllib.unquote(parsed[2]),
+            'PATH_INFO':       self._get_path(parsed),
             'QUERY_STRING':    urlencode(data, doseq=True) or parsed[4],
             'REQUEST_METHOD': 'OPTIONS',
             'wsgi.input':      FakePayload('')
@@ -353,7 +377,7 @@ class Client(object):
 
         response = self.request(**r)
         if follow:
-            response = self._handle_redirects(response)
+            response = self._handle_redirects(response, **extra)
         return response
 
     def put(self, path, data={}, content_type=MULTIPART_CONTENT,
@@ -376,7 +400,7 @@ class Client(object):
         r = {
             'CONTENT_LENGTH': len(post_data),
             'CONTENT_TYPE':   content_type,
-            'PATH_INFO':      urllib.unquote(parsed[2]),
+            'PATH_INFO':      self._get_path(parsed),
             'QUERY_STRING':   query_string or parsed[4],
             'REQUEST_METHOD': 'PUT',
             'wsgi.input':     FakePayload(post_data),
@@ -385,7 +409,7 @@ class Client(object):
 
         response = self.request(**r)
         if follow:
-            response = self._handle_redirects(response)
+            response = self._handle_redirects(response, **extra)
         return response
 
     def delete(self, path, data={}, follow=False, **extra):
@@ -394,7 +418,7 @@ class Client(object):
         """
         parsed = urlparse(path)
         r = {
-            'PATH_INFO':       urllib.unquote(parsed[2]),
+            'PATH_INFO':       self._get_path(parsed),
             'QUERY_STRING':    urlencode(data, doseq=True) or parsed[4],
             'REQUEST_METHOD': 'DELETE',
             'wsgi.input':      FakePayload('')
@@ -403,7 +427,7 @@ class Client(object):
 
         response = self.request(**r)
         if follow:
-            response = self._handle_redirects(response)
+            response = self._handle_redirects(response, **extra)
         return response
 
     def login(self, **credentials):
@@ -458,7 +482,7 @@ class Client(object):
             session.delete(session_key=session_cookie.value)
         self.cookies = SimpleCookie()
 
-    def _handle_redirects(self, response):
+    def _handle_redirects(self, response, **extra):
         "Follows any redirects by requesting responses from the server using GET."
 
         response.redirect_chain = []
@@ -469,7 +493,6 @@ class Client(object):
             redirect_chain = response.redirect_chain
             redirect_chain.append((url, response.status_code))
 
-            extra = {}
             if scheme:
                 extra['wsgi.url_scheme'] = scheme
 
