@@ -13,12 +13,6 @@ import tempfile
 from django.db import models
 from django.core.files.storage import FileSystemStorage
 
-# Python 2.3 doesn't have sorted()
-try:
-    sorted
-except NameError:
-    from django.utils.itercompat import sorted
-
 temp_storage_dir = tempfile.mkdtemp()
 temp_storage = FileSystemStorage(temp_storage_dir)
 
@@ -45,6 +39,9 @@ class Category(models.Model):
 class Writer(models.Model):
     name = models.CharField(max_length=50, help_text='Use both first and last names.')
 
+    class Meta:
+        ordering = ('name',)
+
     def __unicode__(self):
         return self.name
 
@@ -74,7 +71,7 @@ class ImprovedArticleWithParentLink(models.Model):
     article = models.OneToOneField(Article, parent_link=True)
 
 class BetterWriter(Writer):
-    pass
+    score = models.IntegerField()
 
 class WriterProfile(models.Model):
     writer = models.OneToOneField(Writer, primary_key=True)
@@ -107,7 +104,7 @@ try:
         from PIL import Image, _imaging
     except ImportError:
         import Image, _imaging
-    
+
     test_images = True
 
     class ImageFile(models.Model):
@@ -172,12 +169,15 @@ class ArticleStatus(models.Model):
     status = models.CharField(max_length=2, choices=ARTICLE_STATUS_CHAR, blank=True, null=True)
 
 class Inventory(models.Model):
-   barcode = models.PositiveIntegerField(unique=True)
-   parent = models.ForeignKey('self', to_field='barcode', blank=True, null=True)
-   name = models.CharField(blank=False, max_length=20)
+    barcode = models.PositiveIntegerField(unique=True)
+    parent = models.ForeignKey('self', to_field='barcode', blank=True, null=True)
+    name = models.CharField(blank=False, max_length=20)
 
-   def __unicode__(self):
-      return self.name
+    class Meta:
+        ordering = ('name',)
+
+    def __unicode__(self):
+        return self.name
 
 class Book(models.Model):
     title = models.CharField(max_length=40)
@@ -187,6 +187,18 @@ class Book(models.Model):
     class Meta:
         unique_together = ('title', 'author')
 
+class BookXtra(models.Model):
+    isbn = models.CharField(max_length=16, unique=True)
+    suffix1 = models.IntegerField(blank=True, default=0)
+    suffix2 = models.IntegerField(blank=True, default=0)
+
+    class Meta:
+        unique_together = (('suffix1', 'suffix2'))
+        abstract = True
+
+class DerivedBook(Book, BookXtra):
+    pass
+
 class ExplicitPK(models.Model):
     key = models.CharField(max_length=20, primary_key=True)
     desc = models.CharField(max_length=20, blank=True, unique=True)
@@ -205,6 +217,15 @@ class Post(models.Model):
     def __unicode__(self):
         return self.name
 
+class DerivedPost(Post):
+    pass
+
+class BigInt(models.Model):
+    biggie = models.BigIntegerField()
+
+    def __unicode__(self):
+        return unicode(self.biggie)
+
 class MarkupField(models.CharField):
     def __init__(self, *args, **kwargs):
         kwargs["max_length"] = 20
@@ -221,6 +242,12 @@ class CustomFieldForExclusionModel(models.Model):
     name = models.CharField(max_length=10)
     markup = MarkupField()
 
+class FlexibleDatePost(models.Model):
+    title = models.CharField(max_length=50, unique_for_date='posted', blank=True)
+    slug = models.CharField(max_length=50, unique_for_year='posted', blank=True)
+    subtitle = models.CharField(max_length=50, unique_for_month='posted', blank=True)
+    posted = models.DateField(blank=True, null=True)
+
 __test__ = {'API_TESTS': """
 >>> from django import forms
 >>> from django.forms.models import ModelForm, model_to_dict
@@ -307,6 +334,27 @@ Using 'fields' *and* 'exclude'. Not sure why you'd want to do this, but uh,
 >>> CategoryForm.base_fields.keys()
 ['name']
 
+Using 'widgets'
+
+>>> class CategoryForm(ModelForm):
+...
+...     class Meta:
+...         model = Category
+...         fields = ['name', 'url', 'slug']
+...         widgets = {
+...             'name': forms.Textarea,
+...             'url': forms.TextInput(attrs={'class': 'url'})
+...         }
+
+>>> str(CategoryForm()['name'])
+'<textarea id="id_name" rows="10" cols="40" name="name"></textarea>'
+
+>>> str(CategoryForm()['url'])
+'<input id="id_url" type="text" class="url" name="url" maxlength="40" />'
+
+>>> str(CategoryForm()['slug'])
+'<input id="id_slug" type="text" name="slug" maxlength="20" />'
+
 Don't allow more than one 'model' definition in the inheritance hierarchy.
 Technically, it would generate a valid form, but the fact that the resulting
 save method won't deal with multiple objects is likely to trip up people not
@@ -424,8 +472,8 @@ u'entertainment'
 u'Entertainment'
 >>> f.cleaned_data['slug']
 u'entertainment'
->>> obj = f.save()
->>> obj
+>>> c1 = f.save()
+>>> c1
 <Category: Entertainment>
 >>> Category.objects.all()
 [<Category: Entertainment>]
@@ -439,8 +487,8 @@ u'test'
 u"It's a test"
 >>> f.cleaned_data['slug']
 u'its-test'
->>> obj = f.save()
->>> obj
+>>> c2 = f.save()
+>>> c2
 <Category: It's a test>
 >>> Category.objects.order_by('name')
 [<Category: Entertainment>, <Category: It's a test>]
@@ -457,12 +505,12 @@ u'third'
 u'Third test'
 >>> f.cleaned_data['slug']
 u'third-test'
->>> obj = f.save(commit=False)
->>> obj
+>>> c3 = f.save(commit=False)
+>>> c3
 <Category: Third test>
 >>> Category.objects.order_by('name')
 [<Category: Entertainment>, <Category: It's a test>]
->>> obj.save()
+>>> c3.save()
 >>> Category.objects.order_by('name')
 [<Category: Entertainment>, <Category: It's a test>, <Category: Third test>]
 
@@ -487,10 +535,10 @@ Traceback (most recent call last):
 ValueError: The Category could not be created because the data didn't validate.
 
 Create a couple of Writers.
->>> w = Writer(name='Mike Royko')
->>> w.save()
->>> w = Writer(name='Bob Woodward')
->>> w.save()
+>>> w_royko = Writer(name='Mike Royko')
+>>> w_royko.save()
+>>> w_woodward = Writer(name='Bob Woodward')
+>>> w_woodward.save()
 
 ManyToManyFields are represented by a MultipleChoiceField, ForeignKeys and any
 fields with the 'choices' attribute are represented by a ChoiceField.
@@ -504,8 +552,8 @@ fields with the 'choices' attribute are represented by a ChoiceField.
 <tr><th>Pub date:</th><td><input type="text" name="pub_date" /></td></tr>
 <tr><th>Writer:</th><td><select name="writer">
 <option value="" selected="selected">---------</option>
-<option value="1">Mike Royko</option>
-<option value="2">Bob Woodward</option>
+<option value="...">Bob Woodward</option>
+<option value="...">Mike Royko</option>
 </select></td></tr>
 <tr><th>Article:</th><td><textarea rows="10" cols="40" name="article"></textarea></td></tr>
 <tr><th>Status:</th><td><select name="status">
@@ -515,9 +563,9 @@ fields with the 'choices' attribute are represented by a ChoiceField.
 <option value="3">Live</option>
 </select></td></tr>
 <tr><th>Categories:</th><td><select multiple="multiple" name="categories">
-<option value="1">Entertainment</option>
-<option value="2">It&#39;s a test</option>
-<option value="3">Third test</option>
+<option value="...">Entertainment</option>
+<option value="...">It&#39;s a test</option>
+<option value="...">Third test</option>
 </select><br /> Hold down "Control", or "Command" on a Mac, to select more than one.</td></tr>
 
 You can restrict a form to a subset of the complete list of fields
@@ -547,8 +595,9 @@ inserted as 'initial' data in each Field.
 
 >>> art = Article(headline='Test article', slug='test-article', pub_date=datetime.date(1988, 1, 4), writer=w, article='Hello.')
 >>> art.save()
->>> art.id
-1
+>>> art_id_1 = art.id
+>>> art_id_1 is not None
+True
 >>> class TestArticleForm(ModelForm):
 ...     class Meta:
 ...         model = Article
@@ -559,8 +608,8 @@ inserted as 'initial' data in each Field.
 <li>Pub date: <input type="text" name="pub_date" value="1988-01-04" /></li>
 <li>Writer: <select name="writer">
 <option value="">---------</option>
-<option value="1" selected="selected">Mike Royko</option>
-<option value="2">Bob Woodward</option>
+<option value="...">Bob Woodward</option>
+<option value="..." selected="selected">Mike Royko</option>
 </select></li>
 <li>Article: <textarea rows="10" cols="40" name="article">Hello.</textarea></li>
 <li>Status: <select name="status">
@@ -570,17 +619,19 @@ inserted as 'initial' data in each Field.
 <option value="3">Live</option>
 </select></li>
 <li>Categories: <select multiple="multiple" name="categories">
-<option value="1">Entertainment</option>
-<option value="2">It&#39;s a test</option>
-<option value="3">Third test</option>
+<option value="...">Entertainment</option>
+<option value="...">It&#39;s a test</option>
+<option value="...">Third test</option>
 </select>  Hold down "Control", or "Command" on a Mac, to select more than one.</li>
->>> f = TestArticleForm({'headline': u'Test headline', 'slug': 'test-headline', 'pub_date': u'1984-02-06', 'writer': u'1', 'article': 'Hello.'}, instance=art)
+>>> f = TestArticleForm({'headline': u'Test headline', 'slug': 'test-headline', 'pub_date': u'1984-02-06', 'writer': unicode(w_royko.pk), 'article': 'Hello.'}, instance=art)
+>>> f.errors
+{}
 >>> f.is_valid()
 True
 >>> test_art = f.save()
->>> test_art.id
-1
->>> test_art = Article.objects.get(id=1)
+>>> test_art.id == art_id_1
+True
+>>> test_art = Article.objects.get(id=art_id_1)
 >>> test_art.headline
 u'Test headline'
 
@@ -598,9 +649,9 @@ by specifying a 'fields' argument to form_for_instance.
 >>> f.is_valid()
 True
 >>> new_art = f.save()
->>> new_art.id
-1
->>> new_art = Article.objects.get(id=1)
+>>> new_art.id == art_id_1
+True
+>>> new_art = Article.objects.get(id=art_id_1)
 >>> new_art.headline
 u'New headline'
 
@@ -620,8 +671,8 @@ Add some categories and test the many-to-many form output.
 <li>Pub date: <input type="text" name="pub_date" value="1988-01-04" /></li>
 <li>Writer: <select name="writer">
 <option value="">---------</option>
-<option value="1" selected="selected">Mike Royko</option>
-<option value="2">Bob Woodward</option>
+<option value="...">Bob Woodward</option>
+<option value="..." selected="selected">Mike Royko</option>
 </select></li>
 <li>Article: <textarea rows="10" cols="40" name="article">Hello.</textarea></li>
 <li>Status: <select name="status">
@@ -631,21 +682,21 @@ Add some categories and test the many-to-many form output.
 <option value="3">Live</option>
 </select></li>
 <li>Categories: <select multiple="multiple" name="categories">
-<option value="1" selected="selected">Entertainment</option>
-<option value="2">It&#39;s a test</option>
-<option value="3">Third test</option>
+<option value="..." selected="selected">Entertainment</option>
+<option value="...">It&#39;s a test</option>
+<option value="...">Third test</option>
 </select>  Hold down "Control", or "Command" on a Mac, to select more than one.</li>
 
 Initial values can be provided for model forms
->>> f = TestArticleForm(auto_id=False, initial={'headline': 'Your headline here', 'categories': ['1','2']})
+>>> f = TestArticleForm(auto_id=False, initial={'headline': 'Your headline here', 'categories': [str(c1.id), str(c2.id)]})
 >>> print f.as_ul()
 <li>Headline: <input type="text" name="headline" value="Your headline here" maxlength="50" /></li>
 <li>Slug: <input type="text" name="slug" maxlength="50" /></li>
 <li>Pub date: <input type="text" name="pub_date" /></li>
 <li>Writer: <select name="writer">
 <option value="" selected="selected">---------</option>
-<option value="1">Mike Royko</option>
-<option value="2">Bob Woodward</option>
+<option value="...">Bob Woodward</option>
+<option value="...">Mike Royko</option>
 </select></li>
 <li>Article: <textarea rows="10" cols="40" name="article"></textarea></li>
 <li>Status: <select name="status">
@@ -655,27 +706,27 @@ Initial values can be provided for model forms
 <option value="3">Live</option>
 </select></li>
 <li>Categories: <select multiple="multiple" name="categories">
-<option value="1" selected="selected">Entertainment</option>
-<option value="2" selected="selected">It&#39;s a test</option>
-<option value="3">Third test</option>
+<option value="..." selected="selected">Entertainment</option>
+<option value="..." selected="selected">It&#39;s a test</option>
+<option value="...">Third test</option>
 </select>  Hold down "Control", or "Command" on a Mac, to select more than one.</li>
 
 >>> f = TestArticleForm({'headline': u'New headline', 'slug': u'new-headline', 'pub_date': u'1988-01-04',
-...     'writer': u'1', 'article': u'Hello.', 'categories': [u'1', u'2']}, instance=new_art)
+...     'writer': unicode(w_royko.pk), 'article': u'Hello.', 'categories': [unicode(c1.id), unicode(c2.id)]}, instance=new_art)
 >>> new_art = f.save()
->>> new_art.id
-1
->>> new_art = Article.objects.get(id=1)
+>>> new_art.id == art_id_1
+True
+>>> new_art = Article.objects.get(id=art_id_1)
 >>> new_art.categories.order_by('name')
 [<Category: Entertainment>, <Category: It's a test>]
 
 Now, submit form data with no categories. This deletes the existing categories.
 >>> f = TestArticleForm({'headline': u'New headline', 'slug': u'new-headline', 'pub_date': u'1988-01-04',
-...     'writer': u'1', 'article': u'Hello.'}, instance=new_art)
+...     'writer': unicode(w_royko.pk), 'article': u'Hello.'}, instance=new_art)
 >>> new_art = f.save()
->>> new_art.id
-1
->>> new_art = Article.objects.get(id=1)
+>>> new_art.id == art_id_1
+True
+>>> new_art = Article.objects.get(id=art_id_1)
 >>> new_art.categories.all()
 []
 
@@ -684,11 +735,12 @@ Create a new article, with categories, via the form.
 ...     class Meta:
 ...         model = Article
 >>> f = ArticleForm({'headline': u'The walrus was Paul', 'slug': u'walrus-was-paul', 'pub_date': u'1967-11-01',
-...     'writer': u'1', 'article': u'Test.', 'categories': [u'1', u'2']})
+...     'writer': unicode(w_royko.pk), 'article': u'Test.', 'categories': [unicode(c1.id), unicode(c2.id)]})
 >>> new_art = f.save()
->>> new_art.id
-2
->>> new_art = Article.objects.get(id=2)
+>>> art_id_2 = new_art.id
+>>> art_id_2 not in (None, art_id_1)
+True
+>>> new_art = Article.objects.get(id=art_id_2)
 >>> new_art.categories.order_by('name')
 [<Category: Entertainment>, <Category: It's a test>]
 
@@ -697,11 +749,12 @@ Create a new article, with no categories, via the form.
 ...     class Meta:
 ...         model = Article
 >>> f = ArticleForm({'headline': u'The walrus was Paul', 'slug': u'walrus-was-paul', 'pub_date': u'1967-11-01',
-...     'writer': u'1', 'article': u'Test.'})
+...     'writer': unicode(w_royko.pk), 'article': u'Test.'})
 >>> new_art = f.save()
->>> new_art.id
-3
->>> new_art = Article.objects.get(id=3)
+>>> art_id_3 = new_art.id
+>>> art_id_3 not in (None, art_id_1, art_id_2)
+True
+>>> new_art = Article.objects.get(id=art_id_3)
 >>> new_art.categories.all()
 []
 
@@ -711,16 +764,17 @@ The m2m data won't be saved until save_m2m() is invoked on the form.
 ...     class Meta:
 ...         model = Article
 >>> f = ArticleForm({'headline': u'The walrus was Paul', 'slug': 'walrus-was-paul', 'pub_date': u'1967-11-01',
-...     'writer': u'1', 'article': u'Test.', 'categories': [u'1', u'2']})
+...     'writer': unicode(w_royko.pk), 'article': u'Test.', 'categories': [unicode(c1.id), unicode(c2.id)]})
 >>> new_art = f.save(commit=False)
 
 # Manually save the instance
 >>> new_art.save()
->>> new_art.id
-4
+>>> art_id_4 = new_art.id
+>>> art_id_4 not in (None, art_id_1, art_id_2, art_id_3)
+True
 
 # The instance doesn't have m2m data yet
->>> new_art = Article.objects.get(id=4)
+>>> new_art = Article.objects.get(id=art_id_4)
 >>> new_art.categories.all()
 []
 
@@ -739,12 +793,12 @@ existing Category instance.
 >>> cat = Category.objects.get(name='Third test')
 >>> cat
 <Category: Third test>
->>> cat.id
-3
+>>> cat.id == c3.id
+True
 >>> form = ShortCategory({'name': 'Third', 'slug': 'third', 'url': '3rd'}, instance=cat)
 >>> form.save()
 <Category: Third>
->>> Category.objects.get(id=3)
+>>> Category.objects.get(id=c3.id)
 <Category: Third>
 
 Here, we demonstrate that choices for a ForeignKey ChoiceField are determined
@@ -760,8 +814,8 @@ the data in the database when the form is instantiated.
 <li>Pub date: <input type="text" name="pub_date" /></li>
 <li>Writer: <select name="writer">
 <option value="" selected="selected">---------</option>
-<option value="1">Mike Royko</option>
-<option value="2">Bob Woodward</option>
+<option value="...">Bob Woodward</option>
+<option value="...">Mike Royko</option>
 </select></li>
 <li>Article: <textarea rows="10" cols="40" name="article"></textarea></li>
 <li>Status: <select name="status">
@@ -771,11 +825,12 @@ the data in the database when the form is instantiated.
 <option value="3">Live</option>
 </select></li>
 <li>Categories: <select multiple="multiple" name="categories">
-<option value="1">Entertainment</option>
-<option value="2">It&#39;s a test</option>
-<option value="3">Third</option>
+<option value="...">Entertainment</option>
+<option value="...">It&#39;s a test</option>
+<option value="...">Third</option>
 </select>  Hold down "Control", or "Command" on a Mac, to select more than one.</li>
->>> Category.objects.create(name='Fourth', url='4th')
+>>> c4 = Category.objects.create(name='Fourth', url='4th')
+>>> c4
 <Category: Fourth>
 >>> Writer.objects.create(name='Carl Bernstein')
 <Writer: Carl Bernstein>
@@ -785,9 +840,9 @@ the data in the database when the form is instantiated.
 <li>Pub date: <input type="text" name="pub_date" /></li>
 <li>Writer: <select name="writer">
 <option value="" selected="selected">---------</option>
-<option value="1">Mike Royko</option>
-<option value="2">Bob Woodward</option>
-<option value="3">Carl Bernstein</option>
+<option value="...">Bob Woodward</option>
+<option value="...">Carl Bernstein</option>
+<option value="...">Mike Royko</option>
 </select></li>
 <li>Article: <textarea rows="10" cols="40" name="article"></textarea></li>
 <li>Status: <select name="status">
@@ -797,10 +852,10 @@ the data in the database when the form is instantiated.
 <option value="3">Live</option>
 </select></li>
 <li>Categories: <select multiple="multiple" name="categories">
-<option value="1">Entertainment</option>
-<option value="2">It&#39;s a test</option>
-<option value="3">Third</option>
-<option value="4">Fourth</option>
+<option value="...">Entertainment</option>
+<option value="...">It&#39;s a test</option>
+<option value="...">Third</option>
+<option value="...">Fourth</option>
 </select>  Hold down "Control", or "Command" on a Mac, to select more than one.</li>
 
 # ModelChoiceField ############################################################
@@ -809,7 +864,7 @@ the data in the database when the form is instantiated.
 
 >>> f = ModelChoiceField(Category.objects.all())
 >>> list(f.choices)
-[(u'', u'---------'), (1, u'Entertainment'), (2, u"It's a test"), (3, u'Third'), (4, u'Fourth')]
+[(u'', u'---------'), (..., u'Entertainment'), (..., u"It's a test"), (..., u'Third'), (..., u'Fourth')]
 >>> f.clean('')
 Traceback (most recent call last):
 ...
@@ -822,33 +877,34 @@ ValidationError: [u'This field is required.']
 Traceback (most recent call last):
 ...
 ValidationError: [u'Select a valid choice. That choice is not one of the available choices.']
->>> f.clean(3)
+>>> f.clean(c3.id)
 <Category: Third>
->>> f.clean(2)
+>>> f.clean(c2.id)
 <Category: It's a test>
 
 # Add a Category object *after* the ModelChoiceField has already been
 # instantiated. This proves clean() checks the database during clean() rather
 # than caching it at time of instantiation.
->>> Category.objects.create(name='Fifth', url='5th')
+>>> c5 = Category.objects.create(name='Fifth', url='5th')
+>>> c5
 <Category: Fifth>
->>> f.clean(5)
+>>> f.clean(c5.id)
 <Category: Fifth>
 
 # Delete a Category object *after* the ModelChoiceField has already been
 # instantiated. This proves clean() checks the database during clean() rather
 # than caching it at time of instantiation.
 >>> Category.objects.get(url='5th').delete()
->>> f.clean(5)
+>>> f.clean(c5.id)
 Traceback (most recent call last):
 ...
 ValidationError: [u'Select a valid choice. That choice is not one of the available choices.']
 
->>> f = ModelChoiceField(Category.objects.filter(pk=1), required=False)
+>>> f = ModelChoiceField(Category.objects.filter(pk=c1.id), required=False)
 >>> print f.clean('')
 None
 >>> f.clean('')
->>> f.clean('1')
+>>> f.clean(str(c1.id))
 <Category: Entertainment>
 >>> f.clean('100')
 Traceback (most recent call last):
@@ -858,10 +914,10 @@ ValidationError: [u'Select a valid choice. That choice is not one of the availab
 # queryset can be changed after the field is created.
 >>> f.queryset = Category.objects.exclude(name='Fourth')
 >>> list(f.choices)
-[(u'', u'---------'), (1, u'Entertainment'), (2, u"It's a test"), (3, u'Third')]
->>> f.clean(3)
+[(u'', u'---------'), (..., u'Entertainment'), (..., u"It's a test"), (..., u'Third')]
+>>> f.clean(c3.id)
 <Category: Third>
->>> f.clean(4)
+>>> f.clean(c4.id)
 Traceback (most recent call last):
 ...
 ValidationError: [u'Select a valid choice. That choice is not one of the available choices.']
@@ -870,21 +926,21 @@ ValidationError: [u'Select a valid choice. That choice is not one of the availab
 >>> gen_one = list(f.choices)
 >>> gen_two = f.choices
 >>> gen_one[2]
-(2L, u"It's a test")
+(..., u"It's a test")
 >>> list(gen_two)
-[(u'', u'---------'), (1L, u'Entertainment'), (2L, u"It's a test"), (3L, u'Third')]
+[(u'', u'---------'), (..., u'Entertainment'), (..., u"It's a test"), (..., u'Third')]
 
 # check that we can override the label_from_instance method to print custom labels (#4620)
 >>> f.queryset = Category.objects.all()
 >>> f.label_from_instance = lambda obj: "category " + str(obj)
 >>> list(f.choices)
-[(u'', u'---------'), (1L, 'category Entertainment'), (2L, "category It's a test"), (3L, 'category Third'), (4L, 'category Fourth')]
+[(u'', u'---------'), (..., 'category Entertainment'), (..., "category It's a test"), (..., 'category Third'), (..., 'category Fourth')]
 
 # ModelMultipleChoiceField ####################################################
 
 >>> f = ModelMultipleChoiceField(Category.objects.all())
 >>> list(f.choices)
-[(1, u'Entertainment'), (2, u"It's a test"), (3, u'Third'), (4, u'Fourth')]
+[(..., u'Entertainment'), (..., u"It's a test"), (..., u'Third'), (..., u'Fourth')]
 >>> f.clean(None)
 Traceback (most recent call last):
 ...
@@ -893,17 +949,17 @@ ValidationError: [u'This field is required.']
 Traceback (most recent call last):
 ...
 ValidationError: [u'This field is required.']
->>> f.clean([1])
+>>> f.clean([c1.id])
 [<Category: Entertainment>]
->>> f.clean([2])
+>>> f.clean([c2.id])
 [<Category: It's a test>]
->>> f.clean(['1'])
+>>> f.clean([str(c1.id)])
 [<Category: Entertainment>]
->>> f.clean(['1', '2'])
+>>> f.clean([str(c1.id), str(c2.id)])
 [<Category: Entertainment>, <Category: It's a test>]
->>> f.clean([1, '2'])
+>>> f.clean([c1.id, str(c2.id)])
 [<Category: Entertainment>, <Category: It's a test>]
->>> f.clean((1, '2'))
+>>> f.clean((c1.id, str(c2.id)))
 [<Category: Entertainment>, <Category: It's a test>]
 >>> f.clean(['100'])
 Traceback (most recent call last):
@@ -921,16 +977,17 @@ ValidationError: [u'"fail" is not a valid value for a primary key.']
 # Add a Category object *after* the ModelMultipleChoiceField has already been
 # instantiated. This proves clean() checks the database during clean() rather
 # than caching it at time of instantiation.
->>> Category.objects.create(id=6, name='Sixth', url='6th')
+>>> c6 = Category.objects.create(id=6, name='Sixth', url='6th')
+>>> c6
 <Category: Sixth>
->>> f.clean([6])
+>>> f.clean([c6.id])
 [<Category: Sixth>]
 
 # Delete a Category object *after* the ModelMultipleChoiceField has already been
 # instantiated. This proves clean() checks the database during clean() rather
 # than caching it at time of instantiation.
 >>> Category.objects.get(url='6th').delete()
->>> f.clean([6])
+>>> f.clean([c6.id])
 Traceback (most recent call last):
 ...
 ValidationError: [u'Select a valid choice. 6 is not one of the available choices.']
@@ -944,11 +1001,11 @@ ValidationError: [u'Select a valid choice. 6 is not one of the available choices
 Traceback (most recent call last):
 ...
 ValidationError: [u'Select a valid choice. 10 is not one of the available choices.']
->>> f.clean(['3', '10'])
+>>> f.clean([str(c3.id), '10'])
 Traceback (most recent call last):
 ...
 ValidationError: [u'Select a valid choice. 10 is not one of the available choices.']
->>> f.clean(['1', '10'])
+>>> f.clean([str(c1.id), '10'])
 Traceback (most recent call last):
 ...
 ValidationError: [u'Select a valid choice. 10 is not one of the available choices.']
@@ -956,22 +1013,22 @@ ValidationError: [u'Select a valid choice. 10 is not one of the available choice
 # queryset can be changed after the field is created.
 >>> f.queryset = Category.objects.exclude(name='Fourth')
 >>> list(f.choices)
-[(1, u'Entertainment'), (2, u"It's a test"), (3, u'Third')]
->>> f.clean([3])
+[(..., u'Entertainment'), (..., u"It's a test"), (..., u'Third')]
+>>> f.clean([c3.id])
 [<Category: Third>]
->>> f.clean([4])
+>>> f.clean([c4.id])
 Traceback (most recent call last):
 ...
-ValidationError: [u'Select a valid choice. 4 is not one of the available choices.']
->>> f.clean(['3', '4'])
+ValidationError: [u'Select a valid choice. ... is not one of the available choices.']
+>>> f.clean([str(c3.id), str(c4.id)])
 Traceback (most recent call last):
 ...
-ValidationError: [u'Select a valid choice. 4 is not one of the available choices.']
+ValidationError: [u'Select a valid choice. ... is not one of the available choices.']
 
 >>> f.queryset = Category.objects.all()
 >>> f.label_from_instance = lambda obj: "multicategory " + str(obj)
 >>> list(f.choices)
-[(1L, 'multicategory Entertainment'), (2L, "multicategory It's a test"), (3L, 'multicategory Third'), (4L, 'multicategory Fourth')]
+[(..., 'multicategory Entertainment'), (..., "multicategory It's a test"), (..., 'multicategory Third'), (..., 'multicategory Fourth')]
 
 # OneToOneField ###############################################################
 
@@ -987,10 +1044,20 @@ ValidationError: [u'Select a valid choice. 4 is not one of the available choices
 >>> ImprovedArticleWithParentLinkForm.base_fields.keys()
 []
 
->>> bw = BetterWriter(name=u'Joe Better')
+>>> bw = BetterWriter(name=u'Joe Better', score=10)
 >>> bw.save()
 >>> sorted(model_to_dict(bw).keys())
-['id', 'name', 'writer_ptr']
+['id', 'name', 'score', 'writer_ptr']
+
+>>> class BetterWriterForm(ModelForm):
+...     class Meta:
+...         model = BetterWriter
+>>> form = BetterWriterForm({'name': 'Some Name', 'score': 12})
+>>> form.is_valid()
+True
+>>> bw2 = form.save()
+>>> bw2.delete()
+
 
 >>> class WriterProfileForm(ModelForm):
 ...     class Meta:
@@ -999,15 +1066,15 @@ ValidationError: [u'Select a valid choice. 4 is not one of the available choices
 >>> print form.as_p()
 <p><label for="id_writer">Writer:</label> <select name="writer" id="id_writer">
 <option value="" selected="selected">---------</option>
-<option value="1">Mike Royko</option>
-<option value="2">Bob Woodward</option>
-<option value="3">Carl Bernstein</option>
-<option value="4">Joe Better</option>
+<option value="...">Bob Woodward</option>
+<option value="...">Carl Bernstein</option>
+<option value="...">Joe Better</option>
+<option value="...">Mike Royko</option>
 </select></p>
 <p><label for="id_age">Age:</label> <input type="text" name="age" id="id_age" /></p>
 
 >>> data = {
-...     'writer': u'2',
+...     'writer': unicode(w_woodward.pk),
 ...     'age': u'65',
 ... }
 >>> form = WriterProfileForm(data)
@@ -1019,10 +1086,10 @@ ValidationError: [u'Select a valid choice. 4 is not one of the available choices
 >>> print form.as_p()
 <p><label for="id_writer">Writer:</label> <select name="writer" id="id_writer">
 <option value="">---------</option>
-<option value="1">Mike Royko</option>
-<option value="2" selected="selected">Bob Woodward</option>
-<option value="3">Carl Bernstein</option>
-<option value="4">Joe Better</option>
+<option value="..." selected="selected">Bob Woodward</option>
+<option value="...">Carl Bernstein</option>
+<option value="...">Joe Better</option>
+<option value="...">Mike Royko</option>
 </select></p>
 <p><label for="id_age">Age:</label> <input type="text" name="age" value="65" id="id_age" /></p>
 
@@ -1123,7 +1190,6 @@ True
 >>> instance.delete()
 
 # Test the non-required FileField
-
 >>> f = TextFileForm(data={'description': u'Assistance'})
 >>> f.fields['file'].required = False
 >>> f.is_valid()
@@ -1165,6 +1231,28 @@ True
 # Delete the current file since this is not done by Django.
 >>> instance.file.delete()
 >>> instance.delete()
+
+# BigIntegerField ################################################################
+>>> class BigIntForm(forms.ModelForm):
+...     class Meta:
+...         model = BigInt
+...
+>>> bif = BigIntForm({'biggie': '-9223372036854775808'})
+>>> bif.is_valid()
+True
+>>> bif = BigIntForm({'biggie': '-9223372036854775809'})
+>>> bif.is_valid()
+False
+>>> bif.errors
+{'biggie': [u'Ensure this value is greater than or equal to -9223372036854775808.']}
+>>> bif = BigIntForm({'biggie': '9223372036854775807'})
+>>> bif.is_valid()
+True
+>>> bif = BigIntForm({'biggie': '9223372036854775808'})
+>>> bif.is_valid()
+False
+>>> bif.errors
+{'biggie': [u'Ensure this value is less than or equal to 9223372036854775807.']}
 """}
 
 if test_images:
@@ -1356,62 +1444,39 @@ __test__['API_TESTS'] += """
 ...    class Meta:
 ...        model = CommaSeparatedInteger
 
->>> f = CommaSeparatedIntegerForm().fields['field']
->>> f.clean('1,2,3')
-u'1,2,3'
->>> f.clean('1a,2')
-Traceback (most recent call last):
-...
-ValidationError: [u'Enter only digits separated by commas.']
->>> f.clean(',,,,')
-u',,,,'
->>> f.clean('1.2')
-Traceback (most recent call last):
-...
-ValidationError: [u'Enter only digits separated by commas.']
->>> f.clean('1,a,2')
-Traceback (most recent call last):
-...
-ValidationError: [u'Enter only digits separated by commas.']
->>> f.clean('1,,2')
-u'1,,2'
->>> f.clean('1')
-u'1'
-
-# unique/unique_together validation
-
->>> class ProductForm(ModelForm):
-...     class Meta:
-...         model = Product
->>> form = ProductForm({'slug': 'teddy-bear-blue'})
->>> form.is_valid()
+>>> f = CommaSeparatedIntegerForm({'field': '1,2,3'})
+>>> f.is_valid()
 True
->>> obj = form.save()
->>> obj
-<Product: teddy-bear-blue>
->>> form = ProductForm({'slug': 'teddy-bear-blue'})
->>> form.is_valid()
-False
->>> form._errors
-{'slug': [u'Product with this Slug already exists.']}
->>> form = ProductForm({'slug': 'teddy-bear-blue'}, instance=obj)
->>> form.is_valid()
+>>> f.cleaned_data
+{'field': u'1,2,3'}
+>>> f = CommaSeparatedIntegerForm({'field': '1a,2'})
+>>> f.errors
+{'field': [u'Enter only digits separated by commas.']}
+>>> f = CommaSeparatedIntegerForm({'field': ',,,,'})
+>>> f.is_valid()
 True
+>>> f.cleaned_data
+{'field': u',,,,'}
+>>> f = CommaSeparatedIntegerForm({'field': '1.2'})
+>>> f.errors
+{'field': [u'Enter only digits separated by commas.']}
+>>> f = CommaSeparatedIntegerForm({'field': '1,a,2'})
+>>> f.errors
+{'field': [u'Enter only digits separated by commas.']}
+>>> f = CommaSeparatedIntegerForm({'field': '1,,2'})
+>>> f.is_valid()
+True
+>>> f.cleaned_data
+{'field': u'1,,2'}
+>>> f = CommaSeparatedIntegerForm({'field': '1'})
+>>> f.is_valid()
+True
+>>> f.cleaned_data
+{'field': u'1'}
 
-# ModelForm test of unique_together constraint
->>> class PriceForm(ModelForm):
-...     class Meta:
-...         model = Price
->>> form = PriceForm({'price': '6.00', 'quantity': '1'})
->>> form.is_valid()
-True
->>> form.save()
-<Price: 1 for 6.00>
->>> form = PriceForm({'price': '6.00', 'quantity': '1'})
->>> form.is_valid()
-False
->>> form._errors
-{'__all__': [u'Price with this Price and Quantity already exists.']}
+This Price instance generated by this form is not valid because the quantity
+field is required, but the form is valid because the field is excluded from
+the form. This is for backwards compatibility.
 
 >>> class PriceForm(ModelForm):
 ...     class Meta:
@@ -1420,51 +1485,30 @@ False
 >>> form = PriceForm({'price': '6.00'})
 >>> form.is_valid()
 True
+>>> price = form.save(commit=False)
+>>> price.full_clean()
+Traceback (most recent call last):
+  ...
+ValidationError: {'quantity': [u'This field cannot be null.']}
 
-# Unique & unique together with null values
->>> class BookForm(ModelForm):
+The form should not validate fields that it doesn't contain even if they are
+specified using 'fields', not 'exclude'.
 ...     class Meta:
-...        model = Book
->>> w = Writer.objects.get(name='Mike Royko')
->>> form = BookForm({'title': 'I May Be Wrong But I Doubt It', 'author' : w.pk})
->>> form.is_valid()
-True
->>> form.save()
-<Book: Book object>
->>> form = BookForm({'title': 'I May Be Wrong But I Doubt It', 'author' : w.pk})
->>> form.is_valid()
-False
->>> form._errors
-{'__all__': [u'Book with this Title and Author already exists.']}
->>> form = BookForm({'title': 'I May Be Wrong But I Doubt It'})
->>> form.is_valid()
-True
->>> form.save()
-<Book: Book object>
->>> form = BookForm({'title': 'I May Be Wrong But I Doubt It'})
+...         model = Price
+...         fields = ('price',)
+>>> form = PriceForm({'price': '6.00'})
 >>> form.is_valid()
 True
 
-# Test for primary_key being in the form and failing validation.
->>> class ExplicitPKForm(ModelForm):
-...     class Meta:
-...         model = ExplicitPK
-...         fields = ('key', 'desc',)
->>> form = ExplicitPKForm({'key': u'', 'desc': u'' })
->>> form.is_valid()
-False
+The form should still have an instance of a model that is not complete and
+not saved into a DB yet.
 
-# Ensure keys and blank character strings are tested for uniqueness.
->>> form = ExplicitPKForm({'key': u'key1', 'desc': u''})
->>> form.is_valid()
+>>> form.instance.price
+Decimal('6.00')
+>>> form.instance.quantity is None
+True
+>>> form.instance.pk is None
 True
->>> form.save()
-<ExplicitPK: key1>
->>> form = ExplicitPKForm({'key': u'key1', 'desc': u''})
->>> form.is_valid()
-False
->>> sorted(form.errors.items())
-[('__all__', [u'Explicit pk with this Key and Desc already exists.']), ('desc', [u'Explicit pk with this Desc already exists.']), ('key', [u'Explicit pk with this Key already exists.'])]
 
 # Choices on CharField and IntegerField
 >>> class ArticleForm(ModelForm):
@@ -1496,8 +1540,8 @@ ValidationError: [u'Select a valid choice. z is not one of the available choices
 ...     print choice
 (u'', u'---------')
 (86, u'Apple')
-(22, u'Pear')
 (87, u'Core')
+(22, u'Pear')
 
 >>> class InventoryForm(ModelForm):
 ...     class Meta:
@@ -1507,8 +1551,8 @@ ValidationError: [u'Select a valid choice. z is not one of the available choices
 <select name="parent" id="id_parent">
 <option value="">---------</option>
 <option value="86" selected="selected">Apple</option>
-<option value="22">Pear</option>
 <option value="87">Core</option>
+<option value="22">Pear</option>
 </select>
 
 >>> data = model_to_dict(core)
@@ -1531,37 +1575,24 @@ ValidationError: [u'Select a valid choice. z is not one of the available choices
 <tr><th><label for="id_description">Description:</label></th><td><input type="text" name="description" id="id_description" /></td></tr>
 <tr><th><label for="id_url">The URL:</label></th><td><input id="id_url" type="text" name="url" maxlength="40" /></td></tr>
 
-### Validation on unique_for_date
+# to_field_name should also work on ModelMultipleChoiceField ##################
 
->>> p = Post.objects.create(title="Django 1.0 is released", slug="Django 1.0", subtitle="Finally", posted=datetime.date(2008, 9, 3))
->>> class PostForm(ModelForm):
-...     class Meta:
-...         model = Post
+>>> field = ModelMultipleChoiceField(Inventory.objects.all(), to_field_name='barcode')
+>>> for choice in field.choices:
+...     print choice
+(86, u'Apple')
+(87, u'Core')
+(22, u'Pear')
+>>> field.clean([86])
+[<Inventory: Apple>]
 
->>> f = PostForm({'title': "Django 1.0 is released", 'posted': '2008-09-03'})
->>> f.is_valid()
-False
->>> f.errors
-{'title': [u'Title must be unique for Posted date.']}
->>> f = PostForm({'title': "Work on Django 1.1 begins", 'posted': '2008-09-03'})
->>> f.is_valid()
-True
->>> f = PostForm({'title': "Django 1.0 is released", 'posted': '2008-09-04'})
->>> f.is_valid()
-True
->>> f = PostForm({'slug': "Django 1.0", 'posted': '2008-01-01'})
->>> f.is_valid()
-False
->>> f.errors
-{'slug': [u'Slug must be unique for Posted year.']}
->>> f = PostForm({'subtitle': "Finally", 'posted': '2008-09-30'})
->>> f.is_valid()
-False
->>> f.errors
-{'subtitle': [u'Subtitle must be unique for Posted month.']}
->>> f = PostForm({'subtitle': "Finally", "title": "Django 1.0 is released", "slug": "Django 1.0", 'posted': '2008-09-03'}, instance=p)
->>> f.is_valid()
+>>> class SelectInventoryForm(forms.Form):
+...     items = ModelMultipleChoiceField(Inventory.objects.all(), to_field_name='barcode')
+>>> form = SelectInventoryForm({'items': [87, 22]})
+>>> form.is_valid()
 True
+>>> form.cleaned_data
+{'items': [<Inventory: Core>, <Inventory: Pear>]}
 
 # Model field that returns None to exclude itself with explicit fields ########
 
