@@ -1,17 +1,19 @@
 # -*- coding: utf-8 -*-
+import datetime
 import tempfile
 import os
-from django.core.files.storage import FileSystemStorage
-from django.db import models
+
 from django.contrib import admin
+from django.core.files.storage import FileSystemStorage
+from django.contrib.admin.views.main import ChangeList
 from django.core.mail import EmailMessage
+from django.db import models
 from django import forms
 from django.forms.models import BaseModelFormSet
 from django.contrib.auth.models import User
 from django.contrib.contenttypes import generic
 from django.contrib.contenttypes.models import ContentType
 
-
 class Section(models.Model):
     """
     A simple section that links to articles, to test linking to related items
@@ -34,6 +36,7 @@ class Article(models.Model):
     def model_year(self):
         return self.date.year
     model_year.admin_order_field = 'date'
+    model_year.short_description = ''
 
 class Book(models.Model):
     """
@@ -102,6 +105,7 @@ class ArticleAdmin(admin.ModelAdmin):
     def modeladmin_year(self, obj):
         return obj.date.year
     modeladmin_year.admin_order_field = 'date'
+    modeladmin_year.short_description = None
 
 class CustomArticle(models.Model):
     content = models.TextField()
@@ -113,6 +117,7 @@ class CustomArticleAdmin(admin.ModelAdmin):
     """
     change_list_template = 'custom_admin/change_list.html'
     change_form_template = 'custom_admin/change_form.html'
+    add_form_template = 'custom_admin/add_form.html'
     object_history_template = 'custom_admin/object_history.html'
     delete_confirmation_template = 'custom_admin/delete_confirmation.html'
     delete_selected_confirmation_template = 'custom_admin/delete_selected_confirmation.html'
@@ -155,17 +160,27 @@ class Actor(models.Model):
 class Inquisition(models.Model):
     expected = models.BooleanField()
     leader = models.ForeignKey(Actor)
+    country = models.CharField(max_length=20)
+
     def __unicode__(self):
-        return self.expected
+        return u"by %s from %s" % (self.leader, self.country)
+
+class InquisitionAdmin(admin.ModelAdmin):
+    list_display = ('leader', 'country', 'expected')
 
 class Sketch(models.Model):
     title = models.CharField(max_length=100)
     inquisition = models.ForeignKey(Inquisition, limit_choices_to={'leader__name': 'Palin',
                                                                    'leader__age': 27,
+                                                                   'expected': False,
                                                                    })
+
     def __unicode__(self):
         return self.title
 
+class SketchAdmin(admin.ModelAdmin):
+    raw_id_fields = ('inquisition',)
+
 class Fabric(models.Model):
     NG_CHOICES = (
         ('Textured', (
@@ -217,6 +232,14 @@ class PersonAdmin(admin.ModelAdmin):
         return super(PersonAdmin, self).get_changelist_formset(request,
             formset=BasePersonModelFormSet, **kwargs)
 
+class RowLevelChangePermissionModel(models.Model):
+    name = models.CharField(max_length=100, blank=True)
+
+class RowLevelChangePermissionModelAdmin(admin.ModelAdmin):
+    def has_change_permission(self, request, obj=None):
+        """ Only allow changing objects with even id number """
+        return request.user.is_staff and (obj is not None) and (obj.id % 2 == 0)
+
 class Persona(models.Model):
     """
     A simple persona associated with accounts, to test inlining of related
@@ -459,7 +482,69 @@ class CategoryInline(admin.StackedInline):
     model = Category
 
 class CollectorAdmin(admin.ModelAdmin):
-    inlines = [WidgetInline, DooHickeyInline, GrommetInline, WhatsitInline, FancyDoodadInline, CategoryInline]
+    inlines = [
+        WidgetInline, DooHickeyInline, GrommetInline, WhatsitInline,
+        FancyDoodadInline, CategoryInline
+    ]
+
+class Link(models.Model):
+    posted = models.DateField(
+        default=lambda: datetime.date.today() - datetime.timedelta(days=7)
+    )
+    url = models.URLField()
+    post = models.ForeignKey("Post")
+
+
+class LinkInline(admin.TabularInline):
+    model = Link
+    extra = 1
+
+    readonly_fields = ("posted",)
+
+
+class Post(models.Model):
+    title = models.CharField(max_length=100, help_text="Some help text for the title (with unicode ŠĐĆŽćžšđ)")
+    content = models.TextField(help_text="Some help text for the content (with unicode ŠĐĆŽćžšđ)")
+    posted = models.DateField(
+            default=datetime.date.today,
+            help_text="Some help text for the date (with unicode ŠĐĆŽćžšđ)"
+    )
+    public = models.NullBooleanField()
+
+    def awesomeness_level(self):
+        return "Very awesome."
+
+class PostAdmin(admin.ModelAdmin):
+    list_display = ['title', 'public']
+    readonly_fields = ('posted', 'awesomeness_level', 'coolness', 'value', lambda obj: "foo")
+
+    inlines = [
+        LinkInline
+    ]
+
+    def coolness(self, instance):
+        if instance.pk:
+            return "%d amount of cool." % instance.pk
+        else:
+            return "Unkown coolness."
+
+    def value(self, instance):
+        return 1000
+    value.short_description = 'Value in $US'
+
+class Gadget(models.Model):
+    name = models.CharField(max_length=100)
+
+    def __unicode__(self):
+        return self.name
+
+class CustomChangeList(ChangeList):
+    def get_query_set(self):
+        return self.root_query_set.filter(pk=9999) # Does not exist
+
+class GadgetAdmin(admin.ModelAdmin):
+    def get_changelist(self, request, **kwargs):
+        return CustomChangeList
 
 class Villain(models.Model):
     name = models.CharField(max_length=100)
@@ -526,6 +611,16 @@ class CyclicTwo(models.Model):
     def __unicode__(self):
         return self.name
 
+class Topping(models.Model):
+    name = models.CharField(max_length=20)
+
+class Pizza(models.Model):
+    name = models.CharField(max_length=20)
+    toppings = models.ManyToManyField('Topping')
+
+class PizzaAdmin(admin.ModelAdmin):
+    readonly_fields = ('toppings',)
+
 class Album(models.Model):
     owner = models.ForeignKey(User)
     title = models.CharField(max_length=30)
@@ -533,6 +628,99 @@ class Album(models.Model):
 class AlbumAdmin(admin.ModelAdmin):
     list_filter = ['title']
 
+class Employee(Person):
+    code = models.CharField(max_length=20)
+
+class WorkHour(models.Model):
+    datum = models.DateField()
+    employee = models.ForeignKey(Employee)
+
+class WorkHourAdmin(admin.ModelAdmin):
+    list_display = ('datum', 'employee')
+    list_filter = ('employee',)
+
+class Reservation(models.Model):
+    start_date = models.DateTimeField()
+    price = models.IntegerField()
+
+
+DRIVER_CHOICES = (
+    (u'bill', 'Bill G'),
+    (u'steve', 'Steve J'),
+)
+
+RESTAURANT_CHOICES = (
+    (u'indian', u'A Taste of India'),
+    (u'thai', u'Thai Pography'),
+    (u'pizza', u'Pizza Mama'),
+)
+
+class FoodDelivery(models.Model):
+    reference = models.CharField(max_length=100)
+    driver = models.CharField(max_length=100, choices=DRIVER_CHOICES, blank=True)
+    restaurant = models.CharField(max_length=100, choices=RESTAURANT_CHOICES, blank=True)
+
+    class Meta:
+        unique_together = (("driver", "restaurant"),)
+
+class FoodDeliveryAdmin(admin.ModelAdmin):
+    list_display=('reference', 'driver', 'restaurant')
+    list_editable = ('driver', 'restaurant')
+
+class Paper(models.Model):
+    title = models.CharField(max_length=30)
+    author = models.CharField(max_length=30, blank=True, null=True)
+
+class CoverLetter(models.Model):
+    author = models.CharField(max_length=30)
+    date_written = models.DateField(null=True, blank=True)
+
+    def __unicode__(self):
+        return self.author
+
+class PaperAdmin(admin.ModelAdmin):
+    """
+    A ModelAdin with a custom queryset() method that uses only(), to test
+    verbose_name display in messages shown after adding Paper instances.
+    """
+
+    def queryset(self, request):
+        return super(PaperAdmin, self).queryset(request).only('title')
+
+class CoverLetterAdmin(admin.ModelAdmin):
+    """
+    A ModelAdin with a custom queryset() method that uses only(), to test
+    verbose_name display in messages shown after adding CoverLetter instances.
+    Note that the CoverLetter model defines a __unicode__ method.
+    """
+
+    def queryset(self, request):
+        #return super(CoverLetterAdmin, self).queryset(request).only('author')
+        return super(CoverLetterAdmin, self).queryset(request).defer('date_written')
+
+class Story(models.Model):
+    title = models.CharField(max_length=100)
+    content = models.TextField()
+
+class StoryForm(forms.ModelForm):
+    class Meta:
+        widgets = {'title': forms.HiddenInput}
+
+class StoryAdmin(admin.ModelAdmin):
+    list_display = ('id', 'title', 'content')
+    list_display_links = ('title',) # 'id' not in list_display_links
+    list_editable = ('content', )
+    form = StoryForm
+
+class OtherStory(models.Model):
+    title = models.CharField(max_length=100)
+    content = models.TextField()
+
+class OtherStoryAdmin(admin.ModelAdmin):
+    list_display = ('id', 'title', 'content')
+    list_display_links = ('title', 'id') # 'id' in list_display_links
+    list_editable = ('content', )
+
 admin.site.register(Article, ArticleAdmin)
 admin.site.register(CustomArticle, CustomArticleAdmin)
 admin.site.register(Section, save_as=True, inlines=[ArticleInline])
@@ -540,8 +728,8 @@ admin.site.register(ModelWithStringPrimaryKey)
 admin.site.register(Color)
 admin.site.register(Thing, ThingAdmin)
 admin.site.register(Actor)
-admin.site.register(Inquisition)
-admin.site.register(Sketch)
+admin.site.register(Inquisition, InquisitionAdmin)
+admin.site.register(Sketch, SketchAdmin)
 admin.site.register(Person, PersonAdmin)
 admin.site.register(Persona, PersonaAdmin)
 admin.site.register(Subscriber, SubscriberAdmin)
@@ -559,12 +747,22 @@ admin.site.register(Recommendation, RecommendationAdmin)
 admin.site.register(Recommender)
 admin.site.register(Collector, CollectorAdmin)
 admin.site.register(Category, CategoryAdmin)
+admin.site.register(Post, PostAdmin)
+admin.site.register(Gadget, GadgetAdmin)
 admin.site.register(Villain)
 admin.site.register(SuperVillain)
 admin.site.register(Plot)
 admin.site.register(PlotDetails)
 admin.site.register(CyclicOne)
 admin.site.register(CyclicTwo)
+admin.site.register(WorkHour, WorkHourAdmin)
+admin.site.register(Reservation)
+admin.site.register(FoodDelivery, FoodDeliveryAdmin)
+admin.site.register(RowLevelChangePermissionModel, RowLevelChangePermissionModelAdmin)
+admin.site.register(Paper, PaperAdmin)
+admin.site.register(CoverLetter, CoverLetterAdmin)
+admin.site.register(Story, StoryAdmin)
+admin.site.register(OtherStory, OtherStoryAdmin)
 
 # We intentionally register Promo and ChapterXtra1 but not Chapter nor ChapterXtra2.
 # That way we cover all four cases:
@@ -578,4 +776,6 @@ admin.site.register(CyclicTwo)
 admin.site.register(Book, inlines=[ChapterInline])
 admin.site.register(Promo)
 admin.site.register(ChapterXtra1)
-admin.site.register(Album, AlbumAdmin)
\ No newline at end of file
+admin.site.register(Pizza, PizzaAdmin)
+admin.site.register(Topping)
+admin.site.register(Album, AlbumAdmin)
