@@ -3,13 +3,17 @@
 Regression tests for the Test Client, especially the customized assertions.
 """
 import os
-from django.conf import settings
 
-from django.test import Client, TestCase
-from django.test.utils import ContextList
-from django.core.urlresolvers import reverse
+from django.conf import settings
 from django.core.exceptions import SuspiciousOperation
-from django.template import TemplateDoesNotExist, TemplateSyntaxError, Context
+from django.core.urlresolvers import reverse
+from django.template import (TemplateDoesNotExist, TemplateSyntaxError,
+    Context, loader)
+import django.template.context
+from django.test import TestCase, Client
+from django.test.client import encode_file
+from django.test.utils import ContextList
+
 
 class AssertContainsTests(TestCase):
     def setUp(self):
@@ -30,40 +34,86 @@ class AssertContainsTests(TestCase):
         self.assertContains(response, 'twice')
         self.assertContains(response, 'twice', 2)
 
+        try:
+            self.assertContains(response, 'text', status_code=999)
+        except AssertionError, e:
+            self.assertEquals(str(e), "Couldn't retrieve page: Response code was 200 (expected 999)")
+        try:
+            self.assertContains(response, 'text', status_code=999, msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Couldn't retrieve page: Response code was 200 (expected 999)")
+
+        try:
+            self.assertNotContains(response, 'text', status_code=999)
+        except AssertionError, e:
+            self.assertEquals(str(e), "Couldn't retrieve page: Response code was 200 (expected 999)")
+        try:
+            self.assertNotContains(response, 'text', status_code=999, msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Couldn't retrieve page: Response code was 200 (expected 999)")
+
         try:
             self.assertNotContains(response, 'once')
         except AssertionError, e:
             self.assertEquals(str(e), "Response should not contain 'once'")
+        try:
+            self.assertNotContains(response, 'once', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Response should not contain 'once'")
 
         try:
             self.assertContains(response, 'never', 1)
         except AssertionError, e:
             self.assertEquals(str(e), "Found 0 instances of 'never' in response (expected 1)")
+        try:
+            self.assertContains(response, 'never', 1, msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Found 0 instances of 'never' in response (expected 1)")
 
         try:
             self.assertContains(response, 'once', 0)
         except AssertionError, e:
             self.assertEquals(str(e), "Found 1 instances of 'once' in response (expected 0)")
+        try:
+            self.assertContains(response, 'once', 0, msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Found 1 instances of 'once' in response (expected 0)")
 
         try:
             self.assertContains(response, 'once', 2)
         except AssertionError, e:
             self.assertEquals(str(e), "Found 1 instances of 'once' in response (expected 2)")
+        try:
+            self.assertContains(response, 'once', 2, msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Found 1 instances of 'once' in response (expected 2)")
 
         try:
             self.assertContains(response, 'twice', 1)
         except AssertionError, e:
             self.assertEquals(str(e), "Found 2 instances of 'twice' in response (expected 1)")
+        try:
+            self.assertContains(response, 'twice', 1, msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Found 2 instances of 'twice' in response (expected 1)")
 
         try:
             self.assertContains(response, 'thrice')
         except AssertionError, e:
             self.assertEquals(str(e), "Couldn't find 'thrice' in response")
+        try:
+            self.assertContains(response, 'thrice', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Couldn't find 'thrice' in response")
 
         try:
             self.assertContains(response, 'thrice', 3)
         except AssertionError, e:
             self.assertEquals(str(e), "Found 0 instances of 'thrice' in response (expected 3)")
+        try:
+            self.assertContains(response, 'thrice', 3, msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Found 0 instances of 'thrice' in response (expected 3)")
 
     def test_unicode_contains(self):
         "Unicode characters can be found in template context"
@@ -79,6 +129,7 @@ class AssertContainsTests(TestCase):
         self.assertNotContains(r, u'はたけ')
         self.assertNotContains(r, '\xe3\x81\xaf\xe3\x81\x9f\xe3\x81\x91'.decode('utf-8'))
 
+
 class AssertTemplateUsedTests(TestCase):
     fixtures = ['testdata.json']
 
@@ -94,21 +145,35 @@ class AssertTemplateUsedTests(TestCase):
         except AssertionError, e:
             self.assertEquals(str(e), "No templates used to render the response")
 
+        try:
+            self.assertTemplateUsed(response, 'GET Template', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: No templates used to render the response")
+
     def test_single_context(self):
         "Template assertions work when there is a single context"
         response = self.client.get('/test_client/post_view/', {})
 
-        #
         try:
             self.assertTemplateNotUsed(response, 'Empty GET Template')
         except AssertionError, e:
             self.assertEquals(str(e), "Template 'Empty GET Template' was used unexpectedly in rendering the response")
 
+        try:
+            self.assertTemplateNotUsed(response, 'Empty GET Template', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Template 'Empty GET Template' was used unexpectedly in rendering the response")
+
         try:
             self.assertTemplateUsed(response, 'Empty POST Template')
         except AssertionError, e:
             self.assertEquals(str(e), "Template 'Empty POST Template' was not a template used to render the response. Actual template(s) used: Empty GET Template")
 
+        try:
+            self.assertTemplateUsed(response, 'Empty POST Template', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Template 'Empty POST Template' was not a template used to render the response. Actual template(s) used: Empty GET Template")
+
     def test_multiple_context(self):
         "Template assertions work when there are multiple contexts"
         post_data = {
@@ -145,6 +210,11 @@ class AssertRedirectsTests(TestCase):
         except AssertionError, e:
             self.assertEquals(str(e), "Response didn't redirect as expected: Response code was 301 (expected 302)")
 
+        try:
+            self.assertRedirects(response, '/test_client/get_view/', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Response didn't redirect as expected: Response code was 301 (expected 302)")
+
     def test_lost_query(self):
         "An assertion is raised if the redirect location doesn't preserve GET parameters"
         response = self.client.get('/test_client/redirect_view/', {'var': 'value'})
@@ -153,6 +223,11 @@ class AssertRedirectsTests(TestCase):
         except AssertionError, e:
             self.assertEquals(str(e), "Response redirected to 'http://testserver/test_client/get_view/?var=value', expected 'http://testserver/test_client/get_view/'")
 
+        try:
+            self.assertRedirects(response, '/test_client/get_view/', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Response redirected to 'http://testserver/test_client/get_view/?var=value', expected 'http://testserver/test_client/get_view/'")
+
     def test_incorrect_target(self):
         "An assertion is raised if the response redirects to another target"
         response = self.client.get('/test_client/permanent_redirect_view/')
@@ -171,6 +246,12 @@ class AssertRedirectsTests(TestCase):
         except AssertionError, e:
             self.assertEquals(str(e), "Couldn't retrieve redirection page '/test_client/permanent_redirect_view/': response code was 301 (expected 200)")
 
+        try:
+            # The redirect target responds with a 301 code, not 200
+            self.assertRedirects(response, 'http://testserver/test_client/permanent_redirect_view/', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Couldn't retrieve redirection page '/test_client/permanent_redirect_view/': response code was 301 (expected 200)")
+
     def test_redirect_chain(self):
         "You can follow a redirect chain of multiple redirects"
         response = self.client.get('/test_client_regress/redirects/further/more/', {}, follow=True)
@@ -262,6 +343,11 @@ class AssertRedirectsTests(TestCase):
         except AssertionError, e:
             self.assertEquals(str(e), "Response didn't redirect as expected: Response code was 200 (expected 302)")
 
+        try:
+            self.assertRedirects(response, '/test_client/get_view/', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Response didn't redirect as expected: Response code was 200 (expected 302)")
+
     def test_redirect_on_non_redirect_page(self):
         "An assertion is raised if the original page couldn't be retrieved as expected"
         # This page will redirect with code 301, not 302
@@ -271,6 +357,11 @@ class AssertRedirectsTests(TestCase):
         except AssertionError, e:
             self.assertEquals(str(e), "Response didn't redirect as expected: Response code was 200 (expected 302)")
 
+        try:
+            self.assertRedirects(response, '/test_client/get_view/', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEquals(str(e), "abc: Response didn't redirect as expected: Response code was 200 (expected 302)")
+
 
 class AssertFormErrorTests(TestCase):
     def test_unknown_form(self):
@@ -290,6 +381,10 @@ class AssertFormErrorTests(TestCase):
             self.assertFormError(response, 'wrong_form', 'some_field', 'Some error.')
         except AssertionError, e:
             self.assertEqual(str(e), "The form 'wrong_form' was not used to render the response")
+        try:
+            self.assertFormError(response, 'wrong_form', 'some_field', 'Some error.', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEqual(str(e), "abc: The form 'wrong_form' was not used to render the response")
 
     def test_unknown_field(self):
         "An assertion is raised if the field name is unknown"
@@ -308,6 +403,10 @@ class AssertFormErrorTests(TestCase):
             self.assertFormError(response, 'form', 'some_field', 'Some error.')
         except AssertionError, e:
             self.assertEqual(str(e), "The form 'form' in context 0 does not contain the field 'some_field'")
+        try:
+            self.assertFormError(response, 'form', 'some_field', 'Some error.', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEqual(str(e), "abc: The form 'form' in context 0 does not contain the field 'some_field'")
 
     def test_noerror_field(self):
         "An assertion is raised if the field doesn't have any errors"
@@ -326,6 +425,10 @@ class AssertFormErrorTests(TestCase):
             self.assertFormError(response, 'form', 'value', 'Some error.')
         except AssertionError, e:
             self.assertEqual(str(e), "The field 'value' on form 'form' in context 0 contains no errors")
+        try:
+            self.assertFormError(response, 'form', 'value', 'Some error.', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEqual(str(e), "abc: The field 'value' on form 'form' in context 0 contains no errors")
 
     def test_unknown_error(self):
         "An assertion is raised if the field doesn't contain the provided error"
@@ -344,6 +447,10 @@ class AssertFormErrorTests(TestCase):
             self.assertFormError(response, 'form', 'email', 'Some error.')
         except AssertionError, e:
             self.assertEqual(str(e), "The field 'email' on form 'form' in context 0 does not contain the error 'Some error.' (actual errors: [u'Enter a valid e-mail address.'])")
+        try:
+            self.assertFormError(response, 'form', 'email', 'Some error.', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEqual(str(e), "abc: The field 'email' on form 'form' in context 0 does not contain the error 'Some error.' (actual errors: [u'Enter a valid e-mail address.'])")
 
     def test_unknown_nonfield_error(self):
         """
@@ -365,6 +472,10 @@ class AssertFormErrorTests(TestCase):
             self.assertFormError(response, 'form', None, 'Some error.')
         except AssertionError, e:
             self.assertEqual(str(e), "The form 'form' in context 0 does not contain the non-field error 'Some error.' (actual errors: )")
+        try:
+            self.assertFormError(response, 'form', None, 'Some error.', msg_prefix='abc')
+        except AssertionError, e:
+            self.assertEqual(str(e), "abc: The form 'form' in context 0 does not contain the non-field error 'Some error.' (actual errors: )")
 
 class LoginTests(TestCase):
     fixtures = ['testdata']
@@ -375,7 +486,7 @@ class LoginTests(TestCase):
         # Create a second client, and log in.
         c = Client()
         login = c.login(username='testclient', password='password')
-        self.failUnless(login, 'Could not log in')
+        self.assertTrue(login, 'Could not log in')
 
         # Get a redirection page with the second client.
         response = c.get("/test_client_regress/login_protected_redirect_view/")
@@ -399,7 +510,7 @@ class SessionEngineTests(TestCase):
     def test_login(self):
         "A session engine that modifies the session key can be used to log in"
         login = self.client.login(username='testclient', password='password')
-        self.failUnless(login, 'Could not log in')
+        self.assertTrue(login, 'Could not log in')
 
         # Try to access a login protected page.
         response = self.client.get("/test_client/login_protected_view/")
@@ -438,7 +549,7 @@ class ExceptionTests(TestCase):
         "#5836 - A stale user exception isn't re-raised by the test client."
 
         login = self.client.login(username='testclient',password='password')
-        self.failUnless(login, 'Could not log in')
+        self.assertTrue(login, 'Could not log in')
         try:
             response = self.client.get("/test_client_regress/staff_only/")
             self.fail("General users should not be able to visit this page")
@@ -449,7 +560,7 @@ class ExceptionTests(TestCase):
 
         # This next operation should be successful; if it isn't we have a problem.
         login = self.client.login(username='staff', password='password')
-        self.failUnless(login, 'Could not log in')
+        self.assertTrue(login, 'Could not log in')
         try:
             self.client.get("/test_client_regress/staff_only/")
         except SuspiciousOperation:
@@ -457,6 +568,11 @@ class ExceptionTests(TestCase):
 
 class TemplateExceptionTests(TestCase):
     def setUp(self):
+        # Reset the loaders so they don't try to render cached templates.
+        if loader.template_source_loaders is not None:
+            for template_loader in loader.template_source_loaders:
+                if hasattr(template_loader, 'reset'):
+                    template_loader.reset()
         self.old_templates = settings.TEMPLATE_DIRS
         settings.TEMPLATE_DIRS = ()
 
@@ -506,19 +622,45 @@ class ContextTests(TestCase):
         "Context variables can be retrieved from a single context"
         response = self.client.get("/test_client_regress/request_data/", data={'foo':'whiz'})
         self.assertEqual(response.context.__class__, Context)
+        self.assertTrue('get-foo' in response.context)
         self.assertEqual(response.context['get-foo'], 'whiz')
         self.assertEqual(response.context['request-foo'], 'whiz')
         self.assertEqual(response.context['data'], 'sausage')
 
+        try:
+            response.context['does-not-exist']
+            self.fail('Should not be able to retrieve non-existent key')
+        except KeyError, e:
+            self.assertEquals(e.args[0], 'does-not-exist')
+
     def test_inherited_context(self):
         "Context variables can be retrieved from a list of contexts"
         response = self.client.get("/test_client_regress/request_data_extended/", data={'foo':'whiz'})
         self.assertEqual(response.context.__class__, ContextList)
         self.assertEqual(len(response.context), 2)
+        self.assertTrue('get-foo' in response.context)
         self.assertEqual(response.context['get-foo'], 'whiz')
         self.assertEqual(response.context['request-foo'], 'whiz')
         self.assertEqual(response.context['data'], 'bacon')
 
+        try:
+            response.context['does-not-exist']
+            self.fail('Should not be able to retrieve non-existent key')
+        except KeyError, e:
+            self.assertEquals(e.args[0], 'does-not-exist')
+
+    def test_15368(self):
+        # Need to insert a context processor that assumes certain things about
+        # the request instance. This triggers a bug caused by some ways of
+        # copying RequestContext.
+        try:
+            django.template.context._standard_context_processors = (lambda request: {'path': request.special_path},)
+            response = self.client.get("/test_client_regress/request_context_view/")
+            self.assertContains(response, 'Path: /test_client_regress/request_context_view/')
+        finally:
+            django.template.context._standard_context_processors = None
+
+
 class SessionTests(TestCase):
     fixtures = ['testdata.json']
 
@@ -541,7 +683,7 @@ class SessionTests(TestCase):
 
         # Log in
         login = self.client.login(username='testclient',password='password')
-        self.failUnless(login, 'Could not log in')
+        self.assertTrue(login, 'Could not log in')
 
         # Session should still contain the modified value
         response = self.client.get('/test_client_regress/check_session/')
@@ -552,7 +694,7 @@ class SessionTests(TestCase):
         """Logout should work whether the user is logged in or not (#9978)."""
         self.client.logout()
         login = self.client.login(username='testclient',password='password')
-        self.failUnless(login, 'Could not log in')
+        self.assertTrue(login, 'Could not log in')
         self.client.logout()
         self.client.logout()
 
@@ -695,3 +837,43 @@ class UnicodePayloadTests(TestCase):
         response = self.client.post("/test_client_regress/parse_unicode_json/", json,
                                     content_type="application/json; charset=koi8-r")
         self.assertEqual(response.content, json.encode('koi8-r'))
+
+class DummyFile(object):
+    def __init__(self, filename):
+        self.name = filename
+    def read(self):
+        return 'TEST_FILE_CONTENT'
+
+class UploadedFileEncodingTest(TestCase):
+    def test_file_encoding(self):
+        encoded_file = encode_file('TEST_BOUNDARY', 'TEST_KEY', DummyFile('test_name.bin'))
+        self.assertEqual('--TEST_BOUNDARY', encoded_file[0])
+        self.assertEqual('Content-Disposition: form-data; name="TEST_KEY"; filename="test_name.bin"', encoded_file[1])
+        self.assertEqual('TEST_FILE_CONTENT', encoded_file[-1])
+
+    def test_guesses_content_type_on_file_encoding(self):
+        self.assertEqual('Content-Type: application/octet-stream',
+                         encode_file('IGNORE', 'IGNORE', DummyFile("file.bin"))[2])
+        self.assertEqual('Content-Type: text/plain',
+                         encode_file('IGNORE', 'IGNORE', DummyFile("file.txt"))[2])
+        self.assertTrue(encode_file('IGNORE', 'IGNORE', DummyFile("file.zip"))[2] in (
+                        'Content-Type: application/x-compress',
+                        'Content-Type: application/x-zip',
+                        'Content-Type: application/x-zip-compressed',
+                        'Content-Type: application/zip'))
+        self.assertEqual('Content-Type: application/octet-stream',
+                         encode_file('IGNORE', 'IGNORE', DummyFile("file.unknown"))[2])
+
+class RequestHeadersTest(TestCase):
+    def test_client_headers(self):
+        "A test client can receive custom headers"
+        response = self.client.get("/test_client_regress/check_headers/", HTTP_X_ARG_CHECK='Testing 123')
+        self.assertEquals(response.content, "HTTP_X_ARG_CHECK: Testing 123")
+        self.assertEquals(response.status_code, 200)
+
+    def test_client_headers_redirect(self):
+        "Test client headers are preserved through redirects"
+        response = self.client.get("/test_client_regress/check_headers_redirect/", follow=True, HTTP_X_ARG_CHECK='Testing 123')
+        self.assertEquals(response.content, "HTTP_X_ARG_CHECK: Testing 123")
+        self.assertRedirects(response, '/test_client_regress/check_headers/',
+            status_code=301, target_status_code=200)
