@@ -7,6 +7,7 @@ from django.utils import simplejson
 from django.utils.encoding import smart_str
 from django.core.serializers.json import DjangoJSONEncoder
 from django.test.client import CONTENT_TYPE_RE
+from django.template import RequestContext
 
 def no_template_view(request):
     "A simple view that expects a GET request, and returns a rendered template"
@@ -24,6 +25,18 @@ def get_view(request):
     return HttpResponse("Hello world")
 get_view = login_required(get_view)
 
+def request_data(request, template='base.html', data='sausage'):
+    "A simple view that returns the request data in the context"
+    return render_to_response(template, {
+        'get-foo':request.GET.get('foo',None),
+        'get-bar':request.GET.get('bar',None),
+        'post-foo':request.POST.get('foo',None),
+        'post-bar':request.POST.get('bar',None),
+        'request-foo':request.REQUEST.get('foo',None),
+        'request-bar':request.REQUEST.get('bar',None),
+        'data': data,
+    })
+
 def view_with_argument(request, name):
     """A view that takes a string argument
 
@@ -50,6 +63,10 @@ def check_session_view(request):
     "A view that reads a session variable"
     return HttpResponse(request.session.get('session_var', 'NO'))
 
+def request_methods_view(request):
+    "A view that responds with the request method"
+    return HttpResponse('request method: %s' % request.method)
+
 def return_unicode(request):
     return render_to_response('unicode.html')
 
@@ -70,3 +87,12 @@ def return_json_file(request):
                             mimetype='application/json; charset=' + charset)
     response['Content-Disposition'] = 'attachment; filename=testfile.json'
     return response
+
+def check_headers(request):
+    "A view that responds with value of the X-ARG-CHECK header"
+    return HttpResponse('HTTP_X_ARG_CHECK: %s' % request.META.get('HTTP_X_ARG_CHECK', 'Undefined'))
+
+def request_context_view(request):
+    # Special attribute that won't be present on a plain HttpRequest
+    request.special_path = request.path
+    return render_to_response('request_context.html', context_instance=RequestContext(request, {}))
